/*
    random.cpp
    $Id: random.cpp,v 1.6 2014/12/28 09:43:33 daichi Exp $

*/
#include <vector>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include "mt.h"
#include "util.h"
#include "random.h"

using namespace std;

/*
 *  main
 *
 */

void
fisher_yates_shuffle (int *v, int n)
{
	int i, j, t;

	for (i = n - 1; i >= 0; i--)
	{
		j = (int)(RANDOM * (i + 1));
		t = v[i];
		v[i] = v[j];
		v[j] = t;
	}
}

int
bernoulli (double p)	/* return 1 with probability p */
{
	double r = RANDOM;
	if (r <= p)
		return 1;
	else
		return 0;
}

int
multinom (vector<double> &v, int K)
{
	int i;
	double r, s;

	if (K < 0) K = v.size();

	for (i = 0, s = 0; i < K; i++)
		s += v[i];
	r = RANDOM * s;
	for (i = 0, s = 0; i < K; i++)
	{
		s += v[i];
		if (r <= s)
			return i;
	}
	/* for rounding error */
	return K - 1;
}

int
lmultinom (vector<double> &v, int K)
{
	static vector<double> p;
	double z = 0;
	int k;

	if (K < 0) K = v.size();
	p.reserve (K);

	for (k = 0; k < K; k++)
		z = logsumexp(z, v[k], (k == 0));
	for (k = 0; k < K; k++)
		p[k] = exp(v[k] - z);

	return multinom(p, K);
}

int
draw_discrete (double *table, int n)
{
	int i;
	double r, s;

	for (i = 0, s = 0; i < n; i++)
		s += table[i];
	r = RANDOM * s;
	for (i = 0, s = 0; i < n; i++)
	{
		s += table[i];
		if (r <= s)
			return i;
	}
	/* for rounding error */
	printf("FINAL!!\n");
	return n - 1;
}

int
draw_discrete_int (int *table, int n)
{
	int i;
	double r, s;

	for (i = 0, s = 0; i < n; i++)
		s += table[i];
	r = RANDOM * s;
	for (i = 0, s = 0; i < n; i++)
	{
		s += table[i];
		if (r <= s)
			return i;
	}
	/* for rounding error */
	return n - 1;
}

int
draw_probability (double *table, int n)
{
	int i;
	double s = 0, r = RANDOM;

	for (i = 0; i < n; i++)
	{
		s += table[i];
		if (r <= s)
			return i;
	}
	/* for rounding error */
	return n - 1;
}

double
sgamrand (double a)
{
	double x, y, z;
	double u, v, w, b, c, e;
	int accept = 0;
	if (a <= 1)
	{
		/* a <= 1. Johnk's generator. Devroye (1986) p.418 */
		e = exprand();
		do {
			x = pow(RANDOM, 1 / a);
			y = pow(RANDOM, 1 / (1 - a));
		} while (x + y > 1);
		return (e * x / (x + y));
	} else {
		/* a > 1. Best's rejection algorithm. Devroye (1986) p.410 */
		b = a - 1;
		c = 3 * a - 0.75;
		while (accept != 1)
		{
			/* generate */
			u = RANDOM;
			v = RANDOM;
			w = u * (1 - u);
			y = sqrt(c / w) * (u - 0.5);
			x = b + y;
			if (x >= 0)
			{
				z = 64 * w * w * w * v * v;
				if (z <= 1 - (2 * y * y) / x)
				{
					accept = 1;
				} else {
					if (log(z) <= 2 * (b * log(x / b) - y))
						accept = 1;
				}
			}
		}
		return x;
	}
}

double
gamrand (double a, double b)
{
	return sgamrand(a) / b;
}

double
exprand (void)
{
	return (- log(RANDOM));
}

double
betarand (double a, double b)
{
	double x = sgamrand(a), y = sgamrand(b);
	return x / (x + y);
}

int
irand (int n)
{
	return int(RANDOM * n);
}

double
lbetad (double x, double a, double b)
{
	return log(betad(x, a, b));
}

#if 0

int
main (int argc, char *argv[])
{
	int i, n = atoi(argv[1]);

	for (i = 0; i < n; i++)
		printf("%lg\n", betarand(1, 4));
	

}
#endif

