/*
    util.cpp
    $Id: util.cpp,v 1.12 2015/03/06 12:06:18 daichi Exp $

 */
#include <cstdio>
#include <cstring>
#include <cmath>
#include <string>
#include <vector>
#include <utility>
#include <boost/tokenizer.hpp>
#include "gamma.h"
#include "util.h"

using namespace std;

double
poch_ (double alpha, int n)
{
	int i;
	double s = 0;

	if (n == 1)
		return log (alpha);
	else {
		for (i = 0; i < n; i++) {
			s += log (alpha + i);
		}
		return s;
	}
}

double				
poch (double alpha, int n)	/* log of Pochhammer function, log Gamma(a+n)/Gamma(a) */
{
	if (n == 0)
		return 0;
	else if (n == 1)
		return log (alpha);
	else if (n > 0)
		return gammaln (alpha + n) - gammaln (alpha);
	else {
		fprintf(stderr, "lpoch: negative argument n = %d.\n", n);
		exit (1);
	}
}


double
logsumexp (double x, double y, int init)
{
    if (init)
	    return y;

    if (isinf(x) && isinf(y) && x < 0 && y < 0)
	    return -inf;

    if (x > y)
	    return x + log(1 + exp(y - x));
    else
	    return y + log(1 + exp(x - y));
}

char *
strconcat (const char *s, const char *t)
{
	static char z[BUFSIZ];
	strcpy(z, s);
	strcat(z, t);
	return strdup(z);
}

int
eprintf (const char *fmt, ...)
{
	int n;
	va_list ap;

	va_start(ap, fmt);
	n = vfprintf(stderr, fmt, ap);
	va_end(ap);
	fflush(stderr);

	return n;
}
	
int
error (const char *fmt, ...)
{
	int n;
	va_list ap;
	
	va_start(ap, fmt);
	n = vfprintf(stderr, fmt, ap);
	va_end(ap);
	fprintf(stderr, "\n");

	return n;
}

int
ntypes (vector<vector<int> > &data)
{
	int ntypes = 0;
	int i, n = data.size();
	vector<int>::iterator it;

	for (i = 0; i < n; i++)
	{
		for (it = data[i].begin(); it != data[i].end(); it++)
		{
			if (*it > ntypes)
				ntypes = *it;
		}
	}

	return ntypes;
}

typedef boost::tokenizer<boost::char_separator<char> > Tokenizer;
typedef Tokenizer::iterator TokIter;
boost::char_separator<char> sep(" \t");

vector<string>
parse_line (string &line)
{
	Tokenizer tok (line, sep);
	vector<string> ss;

	for (TokIter p = tok.begin(); p != tok.end(); p++)
	{
		ss.push_back (*p);
	}

	return ss;
}


#if 0
template<class Vector>
int
ntypes (vector<Vector>& data)
{
	vector<Vector>::iterator s;
	Vector::iterator w;
	int types = 0;

	for (s = data.begin(); s != data.end(); s++)
	{
		for (w = s->begin(); w != s->end(); w++)
		{
			;
		}
	}

	return types;
}
#endif

#if 0
#include <cstdlib>

int
main (int argc, char *argv[])
{
	double x = atof(argv[1]);

	printf("logsumexp(-inf,%lg) = %lg\n", x, logsumexp(-inf, x));

}

#endif

